#library(Scissor)
library(data.table)
library(tidyverse)
setwd("C:/Users/Zia20/OneDrive/01周志佳2023/02生信和基础文章/01前沿基因")
####load package 


#引用包
library("clusterProfiler")
library("org.Hs.eg.db")
library("enrichplot")
library("ggplot2")

pvalueFilter=0.05       #p值过滤条件
qvalueFilter=0.05       #矫正后的p值过滤条件
inputFile="05.monocleDiff.1.txt"      #输入文件名称
setwd("C:\\biowolf\\sCell\\14.GO")        #设置工作目录

#读取输入文件
rt=read.csv("01_diff_marker2.csv")
#rt=read.table(inputFile, header=T, sep="\t", check.names=F)

#定义颜色
colorSel="qvalue"
if(qvalueFilter>0.05){
  colorSel="pvalue"
}

#基因名字转换为基因id
genes=unique(as.vector(rt[,1]))
entrezIDs=mget(genes, org.Hs.egSYMBOL2EG, ifnotfound=NA)
entrezIDs=as.character(entrezIDs)
gene=entrezIDs[entrezIDs!="NA"]        #去除基因id为NA的基因
#gene=gsub("c\\(\"(\\d+)\".*", "\\1", gene)

#GO富集分析
kk=enrichGO(gene=gene, OrgDb=org.Hs.eg.db, pvalueCutoff=1, qvalueCutoff=1, ont="all", readable=T)
GO=as.data.frame(kk)
GO=GO[(GO$pvalue<pvalueFilter & GO$qvalue<qvalueFilter),]
#保存富集结果
write.table(GO, file="02_GO.txt", sep="\t", quote=F, row.names = F)

#定义显示GO的数目
showNum=10
if(nrow(GO)<30){
  showNum=nrow(GO)
}

#柱状图
pdf(file="02_go_barplot.pdf", width=9, height=12)
bar=barplot(kk, drop=TRUE, showCategory=showNum, split="ONTOLOGY", color=colorSel) + facet_grid(ONTOLOGY~., scale='free')
print(bar)
dev.off()

#气泡图
pdf(file="02_gobubble.pdf", width=9, height=12)
bub=dotplot(kk, showCategory=showNum, orderBy="GeneRatio", split="ONTOLOGY", color=colorSel) + facet_grid(ONTOLOGY~., scale='free')
print(bub)
dev.off()



#引用包
library("clusterProfiler")
library("org.Hs.eg.db")
library("enrichplot")
library("ggplot2")


#kegg富集分析
kk=enrichKEGG(gene=gene, organism="hsa", pvalueCutoff=1, qvalueCutoff=1)
KEGG=as.data.frame(kk)
KEGG$geneID=as.character(sapply(KEGG$geneID,function(x)paste(rt$genes[match(strsplit(x,"/")[[1]],as.character(rt$entrezID))],collapse="/")))
KEGG=KEGG[(KEGG$pvalue<pvalueFilter & KEGG$qvalue<qvalueFilter),]
#保存富集结果
write.table(KEGG, file="02_KEGG.txt", sep="\t", quote=F, row.names = F)

#定义显示通路的数目
showNum=30
if(nrow(KEGG)<showNum){
  showNum=nrow(KEGG)
}

#柱状图
pdf(file="02_kegg_barplot.pdf", width=9, height=15)
barplot(kk, drop=TRUE, showCategory=showNum, color=colorSel)
dev.off()

#气泡图
pdf(file="02_kegg_bubble.pdf", width = 9, height = 15)
dotplot(kk, showCategory=showNum, orderBy="GeneRatio", color=colorSel)
dev.off()
######################无监督分群################

wroksapce="3.cluster0521"
dir.create(wroksapce)
#引用包
library(limma)
library(ConsensusClusterPlus)
    #基因列表文件
workDir=paste0(getwd(),"/", wroksapce )  #工作目录
setwd(workDir)       #设置工作目录

#读取输入文件，并对数据进行处理
tcga=fread("E:/01_project/01_scRNA_HCC/in/01数据准备/TCGA.TPM.txt")

tcga=as.data.frame(tcga)
rownames(tcga)=tcga[,1]
tcga=tcga[,-1]
bulk_dataset=as.matrix(tcga)
colnames(bulk_dataset)=substring(colnames(bulk_dataset),1,16)
bulk_dataset=bulk_dataset[,substring(colnames(bulk_dataset),14,15)=="01"]
rt=bulk_dataset
colnames(rt)=substring(colnames(rt),1,12)
#如果数据没有取log2,会自动对数据取log2
qx=as.numeric(quantile(rt, c(0, 0.25, 0.5, 0.75, 0.99, 1.0), na.rm=T))
LogC=( (qx[5]>100) || ( (qx[6]-qx[1])>50 && qx[2]>0) )
if(LogC){
  rt[rt<0]=0
  rt=log2(rt+1)}

#对数据进行矫正
data=normalizeBetweenArrays(rt)
outData=rbind(ID=colnames(data), data)
write.table(outData, file="03_TCGANormalize.txt", sep="\t", quote=F, col.names=F)


#获取基因的表达量
#rt=read.csv("../01_diff_marker2.csv")
############0521修改###########糖基化相关的基因########################
rt=read.csv("../01_diff_marker3.csv")
#rt=read.csv("../in/glycosylation-related_genes.csv",header = F)

#gene=read.table(geneFile, header=F, sep="\t", check.names=F)
sameGene=intersect(as.vector(rt[,1]), rownames(data))
data=data[sameGene,]

#聚类
maxK=9
results=ConsensusClusterPlus(data,
                             maxK=maxK,
                             reps=50,
                             pItem=0.8,
                             pFeature=1,
                             title=workDir,
                             clusterAlg="pam",
                             distance="euclidean",
                             seed=123456,
                             plot="png")


#输出分型结果
clusterNum=2        #分几类，根据判断标准判断
cluster=results[[clusterNum]][["consensusClass"]]
cluster=as.data.frame(cluster)
cluster$cluster=paste0("C", cluster$cluster)
clusterOut=rbind(ID=colnames(cluster), cluster)
write.table(clusterOut, file="cluster.txt", sep="\t", quote=F, col.names=F)

#########################
load("C:/Users/Zia20/OneDrive/01周志佳2023/02生信和基础文章/01前沿基因/TCGA.Rdata")
#引用包
library(survival)
library(survminer)
cli=cli[,-1]
################
#读取输入文件
tcga=fread("E:/01_project/01_scRNA_HCC/in/01数据准备/TCGA.TPM.txt")

tcga=as.data.frame(tcga)
rownames(tcga)=tcga[,1]
tcga=tcga[,-1]
bulk_dataset=as.matrix(tcga)
colnames(bulk_dataset)=substring(colnames(bulk_dataset),1,12)

#cluster=read.table(clusterFile, header=T, sep="\t", check.names=F, row.names=1)
#rownames(cluster)=gsub("(.*?)\\_(.*?)", "\\2", rownames(cluster))
colnames(cli)=c("futime", "fustat")
cli$futime=cli$futime/365
#row.names(cluster)=substring(row.names(cluster),1,12)
#数据合并
sameSample=intersect(row.names(cluster), row.names(cli))
rt=cbind(cli[sameSample,,drop=F], cluster[sameSample,,drop=F])

#生存差异统计
length=length(levels(factor(rt$cluster)))
diff=survdiff(Surv(futime, fustat) ~ cluster, data = rt)
pValue=1-pchisq(diff$chisq, df=length-1)
if(pValue<0.001){
  pValue="p<0.001"
}else{
  pValue=paste0("p=",sprintf("%.03f",pValue))
}
fit <- survfit(Surv(futime, fustat) ~ cluster, data = rt)
#print(surv_median(fit))

#绘制生存曲线
bioCol=c("#0066FF","#FF9900","#FF0000","#6E568C","#7CC767","#223D6C","#D20A13","#FFD121","#088247","#11AA4D")
bioCol=bioCol[1:length]
surPlot=ggsurvplot(fit, 
                   data=rt,
                   conf.int=F,
                   pval=pValue,
                   pval.size=6,
                   legend.title="Cluster",
                   legend.labs=levels(factor(rt[,"cluster"])),
                   legend = c(0.8, 0.8),
                   font.legend=10,
                   xlab="Time(years)",
                   break.time.by = 1,
                   palette = bioCol,
                   surv.median.line = "hv",
                   risk.table=T,
                   cumevents=F,
                   risk.table.height=.25)
pdf(file="survival.pdf",onefile = FALSE,width=7,height=5.5)
print(surPlot)
dev.off()
#########################

######Video source: https://ke.biowolf.cn
######生信自学网: https://www.biowolf.cn/
######微信公众号：biowolf_cn
######合作邮箱：biowolf@foxmail.com
######答疑微信: 18520221056

######Video source: https://ke.biowolf.cn
######生信自学网: https://www.biowolf.cn/
######微信公众号：biowolf_cn
######合作邮箱：biowolf@foxmail.com
######答疑微信: 18520221056

#install.packages("ggplot2")


library(ggplot2) 
#设置工作目录

#读取输入文件
clinical=read.csv("09_type.csv")
cluster=read.table("cluster.txt", header=T, sep="\t", check.names=F, row.names=1)
rownames(clinical)=clinical[,1]
#合并数据
sameSample=intersect(row.names(clinical), row.names(cluster))
clinical=clinical[sameSample,,drop=F]
cluster=cluster[sameSample,,drop=F]

clinical=clinical[,c(2,3,4,5,6,7,8,9,11,12,13,14)]
load("../02_wgcna_clinc.Rdata")
rownames(clinical)==rownames(tcga_sur)
xixi=intersect(rownames(clinical),rownames(tcga_sur))
clinical=clinical[xixi,]
tcga_sur=tcga_sur[xixi,]
clinical=cbind(clinical,Glycosylation=tcga_sur$Group)
data=cbind(clinical, cluster)
#绘制柱状图
for(i in colnames(data)[1:(ncol(data))-1]){
  rt=data[,c("cluster",i)]
  colnames(rt)=c("Cluster","type")
  #统计检验
  tableStat=table(rt)
  stat=chisq.test(tableStat)
  pvalue=stat$p.value
  if(pvalue<0.001){
    pvalue="p<0.001"
  }else{
    pvalue=paste0("p=",sprintf("%.03f",pvalue))
  }
  #绘制柱状图
  p=ggplot(rt, aes(Cluster)) + 
    geom_bar(aes(fill=type), position="fill")+
    labs(x = 'Cluster',y = '',title=paste0(i," (",pvalue,")"))+
    guides(fill = guide_legend(title =i))+
    theme_bw()+
    theme(plot.title = element_text(hjust = 0.5))
  pdf(file=paste0("cliCor.",i,".pdf"),width=4.5,height=6)	
  print(p)
  dev.off()
}

#################################
######Video source: https://ke.biowolf.cn


#引用包
library(limma)
library(reshape2)
library(ggpubr)
expFile="03_TCGANormalize.txt"            #表达数据文件
geneFile="05.monocleDiff.2.txt"       #基因列表文件
cluFile="cluster.txt"                 #分型结果文件
   #设置工作目录

#读取输入文件，并对数据进行处理
rt=read.table(expFile, header=T, sep="\t", check.names=F)
rt=as.matrix(rt)
rownames(rt)=rt[,1]
exp=rt[,2:ncol(rt)]
dimnames=list(rownames(exp),colnames(exp))
data=matrix(as.numeric(as.matrix(exp)),nrow=nrow(exp),dimnames=dimnames)
data=avereps(data)

#获取基因的表达量
geneRT=read.csv("../01_diff_marker3.csv")
up=geneRT[geneRT[,3]>0,]
down=geneRT[geneRT[,3]<0,]
sameGene=intersect(as.vector(geneRT[,1]), rownames(data))
data=t(data[sameGene,])

#读取分型的结果文件
cluster=read.table(cluFile, header=T, sep="\t", check.names=F, row.names=1)
sameSample=intersect(row.names(data), row.names(cluster))
rt1=cbind(data[sameSample,,drop=F], cluster[sameSample,,drop=F])

#绘制箱线图
bioCol=c("#FF0000","#6E568C","#7CC767","#223D6C","#D20A13","#FFD121","#088247","#11AA4D")
data=melt(rt1, id.vars=c("cluster"))
colnames(data)=c("Cluster","Gene","Expression")
data$Cluster=factor(data$Cluster, levels=levels(factor(data$Cluster)))
yIndex=gsub(".+\\.(.+?)\\.txt", "\\1", geneFile)

#绘制上调基因的箱线图
data1=data[which(data[,2] %in% as.vector(up[,1])),]
p=ggboxplot(data1, x="Cluster", y="Expression", fill = "Cluster",
            ylab=paste0("Up-regulated gene in diffscRNA-seq ",yIndex), add = "none", xlab="Cluster", palette =bioCol[1:length(levels(factor(data$Cluster)))])
#输出图片文件
pdf(file=paste0(yIndex,".up.pdf"), width=4.5, height=6)
print(p)
dev.off()

#绘制下调基因的箱线图
data2=data[which(data[,2] %in% as.vector(down[,1])),]
p=ggboxplot(data2, x="Cluster", y="Expression", fill = "Cluster",
            ylab=paste0("Down-regulated gene in diffscRNA-seq ",yIndex), add = "none", xlab="Cluster", palette =bioCol[1:length(levels(factor(data$Cluster)))])
#输出图片文件
pdf(file=paste0(yIndex,".down.pdf"), width=4.5, height=6)
print(p)
dev.off()
#############之后的都在文件夹中######################








