######Video source: https://ke.biowolf.cn
######生信自学网: https://www.biowolf.cn/
######微信公众号：biowolf_cn
######合作邮箱：biowolf@foxmail.com
######答疑微信: 18520221056

#install.packages("survival")
#install.packages("caret")
#install.packages("glmnet")
#install.packages("survminer")
#install.packages("timeROC")


#引用包
library(survival)
library(caret)
library(glmnet)
library(survminer)
library(timeROC)

setwd("C:/Users/Zia20/OneDrive/01周志佳2023/02生信和基础文章/01前沿基因/37.IMvigor210")     #设置工作目录
rt=read.table("lasso.SigExp.txt", header=T, sep="\t", check.names=F, row.names=1)     #读取输入文件

#模型构建
multiCox <- coxph(Surv(futime, fustat) ~ ., data = rt)
multiCox=step(multiCox,direction = "both")
multiCoxSum=summary(multiCox)
score=predict(multiCox, type="risk", newdata=rt)
coxGene=rownames(multiCoxSum$coefficients)
coxGene=gsub("`", "", coxGene)

#读取IMvigor210表达数据
vigor=read.table("exp.txt", header=T, sep="\t", check.names=F,stringsAsFactors = F)
vigor=vigor[!duplicated(vigor[,1]),]  
rownames(vigor)=vigor[,1]
vigor=vigor[,-1]
samgene=intersect(rownames(vigor),coxGene1)
vigor=t(vigor[samgene,])

ex <- vigor
qx <- as.numeric(quantile(ex, c(0., 0.25, 0.5, 0.75, 0.99, 1.0), na.rm=T))
LogC <- (qx[5] > 100) ||
  (qx[6]-qx[1] > 50 && qx[2] > 0) ||
  (qx[2] > 0 && qx[2] < 1 && qx[4] > 1 && qx[4] < 2)

## 开始判断
if (LogC) { 
  ex[which(ex <= 0)] <- NaN
  ## 取log2
  vigor <- log2(ex)
  print("log2 transform finished")
}else{
  print("log2 transform not needed")
}




#读取生存数据文件
cli=read.table("time.txt", header=T, sep="\t", check.names=F, row.names=1)
colnames(cli)=c("futime", "fustat")
cli$futime=cli$futime/365
cli$futime=as.numeric(cli$futime)
cli=cli[cli$futime<=3.5,]

#合并IMvigor210数据
sameSample=intersect(row.names(vigor), row.names(cli))
vigorTime=cbind(cli[sameSample,,drop=F], vigor[sameSample,,drop=F])
#vigorTime[,3:ncol(vigorTime)]=vigorTime[,3:ncol(vigorTime)]*median(as.matrix(rt[,coxGene]))/median(as.matrix(vigorTime[,3:ncol(vigorTime)]))
colnames(vigorTime)[3]="SAPCD2"
#输出IMvigor210的风险值
vigorScore=predict(multiCox, type="risk", newdata=vigorTime)
Risk=as.vector(ifelse(vigorScore>median(vigorScore), "high", "low"))
vigorRiskOut=cbind(vigorTime, riskScore=as.vector(vigorScore), Risk)
vigorRiskOut=cbind(id=rownames(vigorRiskOut), vigorRiskOut)
write.table(vigorRiskOut,file="risk.IMvigor.txt",sep="\t",quote=F,row.names=F)

#绘制生存曲线函数
bioSurvival=function(inputFile=null, outFile=null){
  #读取输入文件
  rt=read.table(inputFile, header=T, sep="\t", check.names=F)
  #比较高低风险组生存差异，得到显著性p值
  diff=survdiff(Surv(futime, fustat) ~Risk,data = rt)
  pValue=1-pchisq(diff$chisq,df=1)
  if(pValue<0.001){
    pValue="p<0.001"
  }else{
    pValue=paste0("p=",sprintf("%.03f",pValue))
  }
  fit <- survfit(Surv(futime, fustat) ~ Risk, data = rt)
  
  #绘制生存曲线
  surPlot=ggsurvplot(fit, 
                     data=rt,
                     conf.int=F,
                     pval=pValue,
                     pval.size=6,
                     legend.title="Risk",
                     legend.labs=c("High risk", "Low risk"),
                     xlab="Time(years)",
                     break.time.by = 2,
                     palette=c("red", "blue"),
                     risk.table=F,
                     risk.table.title="",
                     risk.table.col = "strata",
                     risk.table.height=.25)
  pdf(file=outFile,onefile = FALSE,width = 6,height =5)
  print(surPlot)
  dev.off()
}

#定义绘制ROC曲线函数
bioROC=function(inputFile=null, outFile=null){
  #读取输入文件
  rt=read.table(inputFile, header=T, sep="\t", check.names=F)
  #ROC曲线
  ROC_rt=timeROC(T=rt$futime,delta=rt$fustat,
                 marker=rt$riskScore,cause=1,
                 weighting='aalen',
                 times=c(1,2,3),ROC=TRUE)
  pdf(file=outFile, width=5, height=5)
  plot(ROC_rt,time=1,col='green',title=FALSE,lwd=2)
  plot(ROC_rt,time=2,col='blue',add=TRUE,title=FALSE,lwd=2)
  plot(ROC_rt,time=3,col='red',add=TRUE,title=FALSE,lwd=2)
  legend('bottomright',
         c(paste0('AUC at 1 years: ',sprintf("%.03f",ROC_rt$AUC[1])),
           paste0('AUC at 2 years: ',sprintf("%.03f",ROC_rt$AUC[2])),
           paste0('AUC at 3 years: ',sprintf("%.03f",ROC_rt$AUC[3]))),
         col=c("green",'blue','red'),lwd=2,bty = 'n')
  dev.off()
}

#调用函数，绘制生存曲线和ROC曲线
bioSurvival(inputFile="risk.IMvigor.txt", outFile="sur.IMvigor.pdf")
bioROC(inputFile="risk.IMvigor.txt", outFile="ROC.IMvigor.pdf")


######Video source: https://ke.biowolf.cn
######生信自学网: https://www.biowolf.cn/
######微信公众号：biowolf_cn
######合作邮箱：biowolf@foxmail.com
######答疑微信: 18520221056

